﻿//TempControl
//Copyright (C) 2006  Casper Kaae Soenderby (casperkaae'at'gmail.com)

//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.



using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using ZedGraph;


namespace TempControl
{
    public partial class Form1 : Form
    {
        
        
        Probe TempProbe = new Probe();
        //string[] SettingItems = new string[] { "Always active", "Always deactive", "Active when temp. is over", "Active when temp. is under" };
        string[] SettingItems = new string[] { "Altid aktiv", "Altid deaktiv", "Aktiv når temperaturen er over", "Aktiv når temperaturen er under" };
        Thread TempThread;
        int TimerElapsedCounter = 0;
        int PowerThrottleCounter = 0;

        //used to throttle the power on each probe if that is enablet
        // 0 = pins is not active
        // 10 = pin always active / 100 % power
        int[] PowerThrottle = new int[] { 0,0,0,0};

        bool LogProbe1 = false; // sets all the probe "loggers" to false
        bool LogProbe2 = false;
        bool LogProbe3 = false;
        bool LogProbe4 = false;
       
        private double _temp1 = 0.00;
        private double _temp2 = 0.00;
        private double _temp3 = 0.00;
        private double _temp4 = 0.00;
        
        //graph variables
        double x, y1, y2, y3, y4;
 
        PointPairList templist1 = new PointPairList();
        PointPairList templist2 = new PointPairList();
        PointPairList templist3 = new PointPairList();
        PointPairList templist4 = new PointPairList();
                     
        public Form1()
        {
            InitializeComponent();
            PortAccess.Output(ParallelPortAddress(Properties.Settings.Default.ParallelPort), 0);//resets all pins on the circuit
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            CreateNewGraph(zedGraphControl1);
            try
            {
                GeneralstatustoolStripStatusLabel.Text = "Initializing temp-circuit: ";
                TempProbe.Initialize();
                GeneralstatustoolStripStatusLabel.Text = "Initializing temp-circuit: Succeed";
            }
            catch (Exception ex)
            {
                GeneralstatustoolStripStatusLabel.Text = "Initializing temp-circuit: failed";
                MessageBox.Show("Could not initialize Temp Circuit, is it connected to the correct comport");

            }

            CircuitversiontoolStripStatusLabel.Text = "Circuit version: " + TempProbe.VersionInfo;
            TempThread = new Thread(new ThreadStart(TempProbe.ReadTempLoop)); // sets TempThread properties when form1 is loaded, waits for start by user..
            TempThread.IsBackground = true;
            try
            {
                  TempThread.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Could not read Temp. from circuit!, Did it initialize properly?");
            }
            
            
            //Setting comboboxes gets populated with SettingsItems
            SettingcomboBox1.Items.AddRange(SettingItems);
            SettingcomboBox1.SelectedIndex = 1;
            SettingcomboBox4.Items.AddRange(SettingItems);
            SettingcomboBox4.SelectedIndex = 1;
            SettingcomboBox3.Items.AddRange(SettingItems);
            SettingcomboBox3.SelectedIndex = 1;
            SettingcomboBox2.Items.AddRange(SettingItems);
            SettingcomboBox2.SelectedIndex = 1;

            //Sets all probe names to the name i settings
            Probelabel1_1.Text = Properties.Settings.Default.Probe1 + ":";
            Probelabel2_1.Text = Properties.Settings.Default.Probe2 + ":";
            Probelabel3_1.Text = Properties.Settings.Default.Probe3 + ":";
            Probelabel4_1.Text = Properties.Settings.Default.Probe4 + ":";

            //Sets all Threshold temps to the last used ones
            ThresholdnumericUpDown1.Value = Properties.Settings.Default.Threshold1;
            ThresholdnumericUpDown2.Value = Properties.Settings.Default.Threshold2;
            ThresholdnumericUpDown3.Value = Properties.Settings.Default.Threshold3;
            ThresholdnumericUpDown4.Value = Properties.Settings.Default.Threshold4;

            //Set ThrottleTemp to the last used ones
            PowerThrottlenumericUpDown1.Value = (decimal)Properties.Settings.Default.ThrottletempProbe1;
            PowerThrottlenumericUpDown2.Value = (decimal)Properties.Settings.Default.ThrottletempProbe2;
            PowerThrottlenumericUpDown3.Value = (decimal)Properties.Settings.Default.ThrottletempProbe3;
            PowerThrottlenumericUpDown4.Value = (decimal)Properties.Settings.Default.ThrottletempProbe4;




        }
        private void LogEnablebutton1_Click(object sender, EventArgs e)
        {
            if (LogfilenametextBox1.Text != "" && ValidFilename(LogfilenametextBox1.Text) &&
                LogfilenametextBox1.Text != LogfilenametextBox2.Text &&
                LogfilenametextBox1.Text != LogfilenametextBox3.Text &&
                LogfilenametextBox1.Text != LogfilenametextBox4.Text)
            {
                LogProbe1 = true;
                LogEnablebutton1.Enabled = false;
                LogDisablebutton1.Enabled = true;
                LogfilenametextBox1.ReadOnly = true;


            }
            else
            {
                MessageBox.Show("You must enter a valid file name:\r\n" + "Is not 'blank'.\r\n" + "Only contains valid filename charaters\r\n" + "Is not Equal to other log file name.");
            }
        }

        private void LogEnablebutton2_Click(object sender, EventArgs e)
        {
            if (LogfilenametextBox2.Text != "" && ValidFilename(LogfilenametextBox2.Text) &&
            LogfilenametextBox2.Text != LogfilenametextBox1.Text &&
            LogfilenametextBox2.Text != LogfilenametextBox3.Text &&
            LogfilenametextBox2.Text != LogfilenametextBox4.Text)
            {
                LogProbe2 = true;
                LogEnablebutton2.Enabled = false;
                LogDisablebutton2.Enabled = true;
                LogfilenametextBox2.ReadOnly = true;


            }
            else
            {
                MessageBox.Show("You must enter a valid file name:\r\n" + "Is not 'blank'.\r\n" + "Only contains valid filename charaters\r\n" + "Is not Equal to other log file name.");
            }
        }

        private void LogEnablebutton3_Click(object sender, EventArgs e)
        {
            if (LogfilenametextBox3.Text != "" && ValidFilename(LogfilenametextBox3.Text) &&
                LogfilenametextBox3.Text != LogfilenametextBox2.Text &&
                LogfilenametextBox3.Text != LogfilenametextBox1.Text &&
                LogfilenametextBox3.Text != LogfilenametextBox4.Text)
            {
                LogProbe3 = true;
                LogEnablebutton3.Enabled = false;
                LogDisablebutton3.Enabled = true;
                LogfilenametextBox3.ReadOnly = true;
            }
            else
            {
                MessageBox.Show("You must enter a valid file name:\r\n" + "Is not 'blank'.\r\n" + "Only contains valid filename charaters\r\n" + "Is not Equal to other log file name.");
            }
        }

        private void LogEnablebutton4_Click(object sender, EventArgs e)
        {
            if (LogfilenametextBox4.Text != "" && ValidFilename(LogfilenametextBox4.Text) &&
                LogfilenametextBox4.Text != LogfilenametextBox1.Text &&
                LogfilenametextBox4.Text != LogfilenametextBox2.Text &&
                LogfilenametextBox4.Text != LogfilenametextBox3.Text)
            {
                LogProbe4 = true;
                LogEnablebutton4.Enabled = false;
                LogDisablebutton4.Enabled = true;
                LogfilenametextBox4.ReadOnly = true;
            }
            else
            {
                MessageBox.Show("You must enter a valid file name:\r\n" + "Is not 'blank'.\r\n" + "Only contains valid filename charaters\r\n" + "Is not Equal to other log file name.");
            }   
        }

        private void LogDisablebutton1_Click(object sender, EventArgs e)
        {
            LogProbe1 = false;
            LogEnablebutton1.Enabled = true;
            LogDisablebutton1.Enabled = false;
            LogfilenametextBox1.ReadOnly = false;
        }

        private void LogDisablebutton2_Click(object sender, EventArgs e)
        {
            LogProbe2 = false;
            LogEnablebutton2.Enabled = true;
            LogDisablebutton2.Enabled = false;
            LogfilenametextBox2.ReadOnly = false;
        }

        private void LogDisablebutton3_Click(object sender, EventArgs e)
        {
            LogProbe3 = false;
            LogEnablebutton3.Enabled = true;
            LogDisablebutton3.Enabled = false;
            LogfilenametextBox3.ReadOnly = false;
        }

        private void LogDisablebutton4_Click(object sender, EventArgs e)
        {
            LogProbe4 = false;
            LogEnablebutton4.Enabled = true;
            LogDisablebutton4.Enabled = false;
            LogfilenametextBox4.ReadOnly = false;
        }       

        private void GraphshowcheckBox1_CheckedChanged(object sender, EventArgs e)
        {
            GraphShow(zedGraphControl1, GraphshowcheckBox1, 0);
        }

        private void GraphshowcheckBox2_CheckedChanged(object sender, EventArgs e)
        {
            GraphShow(zedGraphControl1, GraphshowcheckBox2, 1);
        }

        private void GraphshowcheckBox3_CheckedChanged(object sender, EventArgs e)
        {
            GraphShow(zedGraphControl1, GraphshowcheckBox3, 2);
        }

        private void GraphshowcheckBox4_CheckedChanged(object sender, EventArgs e)
        {
            GraphShow(zedGraphControl1, GraphshowcheckBox4, 3);
        }

        private void GraphShowThresholdcheckBox1_CheckedChanged(object sender, EventArgs e)
        {
            ThresholdTempLineShow(zedGraphControl1, GraphShowThresholdcheckBox1, 0);
        }

        private void GraphShowThresholdcheckBox2_CheckedChanged(object sender, EventArgs e)
        {
            ThresholdTempLineShow(zedGraphControl1, GraphShowThresholdcheckBox2, 1);
        }

        private void GraphShowThresholdcheckBox3_CheckedChanged(object sender, EventArgs e)
        {
            ThresholdTempLineShow(zedGraphControl1, GraphShowThresholdcheckBox3, 2);
        }

        private void GraphShowThresholdcheckBox4_CheckedChanged(object sender, EventArgs e)
        {
            ThresholdTempLineShow(zedGraphControl1, GraphShowThresholdcheckBox4, 3);
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            About_box aboutbox = new About_box();
            aboutbox.ShowDialog();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Windows.Forms.Application.Exit();
        }

        private void ThresholdnumericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.Threshold1 = ThresholdnumericUpDown1.Value;
            Properties.Settings.Default.Save();
        }

        private void ThresholdnumericUpDown2_ValueChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.Threshold2 = ThresholdnumericUpDown2.Value;
            Properties.Settings.Default.Save();
        }

        private void ThresholdnumericUpDown3_ValueChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.Threshold3 = ThresholdnumericUpDown3.Value;
            Properties.Settings.Default.Save();
        }

        private void ThresholdnumericUpDown4_ValueChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.Threshold4 = ThresholdnumericUpDown4.Value;
            Properties.Settings.Default.Save();
        }

        private void SettingstoolStripMenuItem_Click(object sender, EventArgs e)
        {
            Settings settings = new Settings();
            settings.ShowDialog();
            if (settings.DialogResult == DialogResult.OK)
            {
                Probelabel1_1.Text = Properties.Settings.Default.Probe1 + ":";
                Probelabel2_1.Text = Properties.Settings.Default.Probe2 + ":";
                Probelabel3_1.Text = Properties.Settings.Default.Probe3 + ":";
                Probelabel4_1.Text = Properties.Settings.Default.Probe4 + ":";

                Probelabel1_2.Text = Properties.Settings.Default.Probe1 + ":";
                Probelabel2_2.Text = Properties.Settings.Default.Probe2 + ":";
                Probelabel3_2.Text = Properties.Settings.Default.Probe3 + ":";
                Probelabel4_2.Text = Properties.Settings.Default.Probe4 + ":";

                Graphlabel1.Text = Properties.Settings.Default.Probe1 + " graf:";
                Graphlabel2.Text = Properties.Settings.Default.Probe2 + " graf:";
                Graphlabel3.Text = Properties.Settings.Default.Probe3 + " graf:";
                Graphlabel4.Text = Properties.Settings.Default.Probe4 + " graf:";

                TempProbe.Comport = Properties.Settings.Default.ComPort;

            }
        }

        private void GraphsResetbutton_Click(object sender, EventArgs e)
        {
            templist1.RemoveRange(0, templist1.Count);
            templist2.RemoveRange(0, templist2.Count);
            templist3.RemoveRange(0, templist3.Count);
            templist4.RemoveRange(0, templist4.Count);
        }

        private void timer_Elapsed(object sender, System.Timers.ElapsedEventArgs e) // func that is called every 1000 milisecond
        {
            ProbeTemplabel1.Text = TempProbe.Temp1; //°C
            ProbeTemplabel2.Text = TempProbe.Temp2;
            ProbeTemplabel3.Text = TempProbe.Temp3;
            ProbeTemplabel4.Text = TempProbe.Temp4;

            PowerLevellabel1.Text = (10 * PowerThrottle[0]) + "%";
            PowerLevellabel2.Text = (10 * PowerThrottle[1]) + "%";
            PowerLevellabel3.Text = (10 * PowerThrottle[2]) + "%";
            PowerLevellabel4.Text = (10 * PowerThrottle[3]) + "%";

            _temp1 = double.Parse(TempProbe.Temp1);
            _temp2 = double.Parse(TempProbe.Temp2);
            _temp3 = double.Parse(TempProbe.Temp3);
            _temp4 = double.Parse(TempProbe.Temp4);

            //checks tresholdtemps and take action accordingly
            ThresholdTemp((double)ThresholdnumericUpDown1.Value, (double)ThresholdnumericUpDown2.Value, (double)ThresholdnumericUpDown3.Value, (double)ThresholdnumericUpDown4.Value);
            //updates the graph class with current temps
            GraphUpdate(zedGraphControl1);
            
            if (TimerElapsedCounter == 5)
            {
                templist1.RemoveRange(0, 5);
                templist2.RemoveRange(0, 5);
                templist3.RemoveRange(0, 5);
                templist4.RemoveRange(0, 5);
                GraphInitializinglabel.Visible = false;
                zedGraphControl1.Visible = true;
                GraphSettningsgroupBox.Visible = true;
            }


            // log temp every 10th time the timer_elapsed func is called.
            if (TimerElapsedCounter % 10 == 0) 
            {
                LogProbe();
            }


            ++TimerElapsedCounter; // count how many times the Time_elapsed func. is called. eg if you only want someting to happen every fifth time or so 
        }

        private int Power(int a, int b) // raises double a to the power of b
        {
            if (b < 0) return 0;// we want a positive integer for the exponent
            if (b == 0) return 1;
            else
            {
                int c = 1;
                for (int i = 0; i < b; i++)
                {
                    c *= a;
                }
                return c;
            }
        } 


        //Tests the Threshold temps arccordingly to the settings and activate pins i some fills the criterias
       private void ThresholdTemp(double ttemp1, double ttemp2, double ttemp3, double ttemp4)
       {
            ComboBox[] Comboboxes = new ComboBox[] { SettingcomboBox1, SettingcomboBox2, SettingcomboBox3, SettingcomboBox4 };
            PictureBox[] Pictureboxes = new PictureBox[] { StatuspictureBox1, StatuspictureBox2, StatuspictureBox3, StatuspictureBox4 };
            NumericUpDown[] NumericUpDowns = new NumericUpDown[] { ThresholdnumericUpDown1, ThresholdnumericUpDown2, ThresholdnumericUpDown3, ThresholdnumericUpDown4 };
            double[] ThresholdTemps = new double[] { ttemp1, ttemp2, ttemp3, ttemp4 };
            double[] Temps = new double[] { _temp1, _temp2, _temp3, _temp4 };
            double[] ThrottleTemp = new double[] { Properties.Settings.Default.ThrottletempProbe1, Properties.Settings.Default.ThrottletempProbe2, Properties.Settings.Default.ThrottletempProbe3, Properties.Settings.Default.ThrottletempProbe4 };
            int i = 0; // number of iterations through foreach loop
                       // also used to select right temp          
            
           foreach (ComboBox ComboBox in Comboboxes)
            {
                if (ComboBox.SelectedIndex == 0) //pin always active
                {
                    PowerThrottle[i] = 10;
                    
                    Pictureboxes[i].Image = Properties.Resources.greenlight;
                    NumericUpDowns[i].Enabled = false;
                }

                if (ComboBox.SelectedIndex == 1) //always deactive not needed to change pin, just picture, deactivetes pin if nothing is done 
                {
                    PowerThrottle[i] = 0;
                    Pictureboxes[i].Image = Properties.Resources.redlight;
                    NumericUpDowns[i].Enabled = false;

                }
                if (ComboBox.SelectedIndex == 2) // active when temp is over
                {
                    if (Temps[i] > ThresholdTemps[i])
                    {
                        if ((Temps[i] - ThresholdTemps[i]) < ThrottleTemp[i])
                        {
                            //TODO: Make it a bit more precise
                            PowerThrottle[i] = (int)Math.Ceiling((10*((Temps[i] - ThresholdTemps[i]) / ThrottleTemp[i])));
                        }
                        else
                        {
                            PowerThrottle[i] = 10;
                        }
                        
                        Pictureboxes[i].Image = Properties.Resources.greenlight;
                    }
                    else
                    {
                        Pictureboxes[i].Image = Properties.Resources.redlight;
                        PowerThrottle[i] = 0;
                    }
                    NumericUpDowns[i].Enabled = true;
                }
                if (ComboBox.SelectedIndex == 3) //active when temp is under
                
                {
                    if (Temps[i] < ThresholdTemps[i])
                    {
                        if ((ThresholdTemps[i] - Temps[i]) < ThrottleTemp[i])
                        {
                            //TODO: Make it a bit more precise
                            PowerThrottle[i] = (int)Math.Ceiling((10 * ((ThresholdTemps[i] - Temps[i]) / ThrottleTemp[i])));
                        }
                        else
                        {
                            PowerThrottle[i] = 10;
                        }
                        Pictureboxes[i].Image = Properties.Resources.greenlight;
                    }
                    else
                    {
                        Pictureboxes[i].Image = Properties.Resources.redlight;
                        PowerThrottle[i] = 0;
                    }
                    NumericUpDowns[i].Enabled = true;
                }
                i++;
            }

        }
        private void PowerThrottletimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            
            if(PowerThrottleCounter != 10)
            {
                int activepins = 0;
                int pin = 0;

                foreach (int PinPower in PowerThrottle)
                {
                    if (PinPower > PowerThrottleCounter)
                    {
                        activepins += Power(2, pin);  //pin1 = 2^0 = 1   -if pin2 and pin4 needs to be active the value passed -to portacces.output is pin2+pin4 = 2+8 = 10
                        //pin2 = 2^1 = 2   
                        //pin3 = 2^2 = 4
                        //pin4 = 2^3 = 8
                    }
                    ++pin;
                }
                PortAccess.Output(ParallelPortAddress(Properties.Settings.Default.ParallelPort), activepins);
                // the printer port addresses are: (use the dec number in Portacces.Output())
                //0x3BC(hex) = 956(dec)
                //0x378(hex) = 888(dec) 
                //0x278(hex) = 632(dec)
            
            ++PowerThrottleCounter;
            }
            else
            {
                PowerThrottleCounter = 0;
            }

        }


        private int ParallelPortAddress(string Port)
        {
            if (Port == "LPT1")
            {
                return 888;
            }
            if (Port == "LPT2")
            {
                return 632;
            }
            if (Port == "LPT3")
            {
                return 956;
            }
            return 888;
        }
        private void LogProbe() // log temp to file, name speciied by the user, on probes where logging is enabled
        //How to Read Log File:
        //The it is formattet as Commaormattet text file. 
        //First collum is the actual time the temp was logget
        //Second collumTels how many second that has elapsed since the the program startet.
        //Third Collum is the actual temperature i Celsius degrees.
        {
            bool[] LogProbe = new bool[] { LogProbe1, LogProbe2, LogProbe3, LogProbe4 };
            TextBox[] LogFileName = new TextBox[] { LogfilenametextBox1, LogfilenametextBox2, LogfilenametextBox3, LogfilenametextBox4 };
            string[] Temps = new string[] { TempProbe.Temp1, TempProbe.Temp2, TempProbe.Temp3, TempProbe.Temp4 };
            
            int i = 0;
            foreach (bool logprobe in LogProbe)
            {
            string dtnow = DateTime.Now.ToString();
                if (logprobe) // hence logprobe is a bool
                {  
                    StreamWriter sw = File.AppendText(LogFileName[i].Text + ".txt");
                    sw.WriteLine(dtnow + "," + TimerElapsedCounter + "," + Temps[i]);
                    sw.Close();
                }
            ++i;
            }
        }

        private bool ValidFilename(string filename)
        {

            char[] invalidChars = System.IO.Path.GetInvalidFileNameChars();
            if (filename.IndexOfAny(invalidChars) > -1)
            {
                return false;
            }
        return true;

        }

        private void CreateNewGraph(ZedGraphControl zgc)
        {
            // get a reference to the GraphPane
            GraphPane myPane = zgc.GraphPane;

            // Set the Titles
            myPane.Title.Text = "Temperatur/Tid";
            myPane.XAxis.Title.Text = "Tid(Sekunder)";
            myPane.YAxis.Title.Text = "Temperatur(°C)";
            myPane.YAxis.MajorGrid.IsVisible = true;
            myPane.XAxis.MajorGrid.IsVisible = true;
 
            //Temp Curves
            LineItem myCurve1 = myPane.AddCurve("Probe1",
                  templist1, Color.Red, SymbolType.None);


            LineItem myCurve2 = myPane.AddCurve("Probe2",
                  templist2, Color.Blue, SymbolType.None);


            LineItem myCurve3 = myPane.AddCurve("Probe3",
                 templist3, Color.Green, SymbolType.None);


            LineItem myCurve4 = myPane.AddCurve("Probe4",
                  templist4, Color.Purple, SymbolType.None);
            
            //Threshhold temps

            LineObj threshold1 = new LineObj(Color.Red, 0, 20, 1, 20);
            threshold1.Location.CoordinateFrame = CoordType.XChartFractionYScale;
            threshold1.IsVisible = false;

            LineObj threshold2 = new LineObj(Color.Blue, 0, 19, 1, 19);
            threshold2.Location.CoordinateFrame = CoordType.XChartFractionYScale;
            threshold2.IsVisible = false;

            LineObj threshold3 = new LineObj(Color.Green, 0, 18, 1, 18);
            threshold3.Location.CoordinateFrame = CoordType.XChartFractionYScale;
            threshold3.IsVisible = false;

            LineObj threshold4 = new LineObj(Color.Purple, 0, 17, 1, 17);
            threshold4.Location.CoordinateFrame = CoordType.XChartFractionYScale;
            threshold4.IsVisible = false;

            myPane.GraphObjList.Add(threshold1);
            myPane.GraphObjList.Add(threshold2);
            myPane.GraphObjList.Add(threshold3);
            myPane.GraphObjList.Add(threshold4); 
        }


        private void GraphUpdate(ZedGraphControl zgc)
        {
            GraphPane myPane = zgc.GraphPane;
            
            //updates the temperature curves with current temperature
            x = TimerElapsedCounter;
            y1 = _temp1;
            y2 = _temp2;
            y3 = _temp3;
            y4 = _temp4;
            templist1.Add(x, y1);
            templist2.Add(x, y2);
            templist3.Add(x, y3);
            templist4.Add(x, y4);

            CurveList curves = myPane.CurveList; //temp curves array
            GraphObjList thresholdlines = myPane.GraphObjList; //threshold temp lines array
            double[] ttemps = new double[] { (double)ThresholdnumericUpDown1.Value, 
                                             (double)ThresholdnumericUpDown2.Value,
                                             (double)ThresholdnumericUpDown3.Value,
                                             (double)ThresholdnumericUpDown4.Value};
            //array used to set the curve title to the current title of the probe it 'represents'
            string[] curvetitles = new string[] {Probelabel1_1.Text + " " + ProbeTemplabel1.Text + "°C", 
                                                 Probelabel2_1.Text + " " + ProbeTemplabel2.Text + "°C", 
                                                 Probelabel3_1.Text + " " + ProbeTemplabel3.Text + "°C",
                                                 Probelabel4_1.Text + " " + ProbeTemplabel4.Text + "°C"}; 
            for (int i = 0; i != 4; ++i)
            {
                curves[i].Label.Text = curvetitles[i]; // updates the curvetiteles
                thresholdlines[i].Location.Y1 = ttemps[i]; //sets the y cordinates of the thresholdtemp line to the current selected threshold temperature
            }
            zgc.AxisChange();
            zgc.Invalidate();
        }
        
        
        private void GraphShow(ZedGraphControl zgc,CheckBox myCheckBox, int myGraph)
        {
            zgc.GraphPane.CurveList[myGraph].IsVisible = myCheckBox.Checked;       
        }

        private void ThresholdTempLineShow(ZedGraphControl zgc, CheckBox myCheckBox, int myline)
        {
            zgc.GraphPane.GraphObjList[myline].IsVisible = myCheckBox.Checked;
        }

        private void PowerThrottlenumericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.ThrottletempProbe1 = (double)PowerThrottlenumericUpDown1.Value;
            Properties.Settings.Default.Save();
        }

        private void PowerThrottlenumericUpDown2_ValueChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.ThrottletempProbe2 = (double)PowerThrottlenumericUpDown2.Value;
            Properties.Settings.Default.Save();
        }

        private void PowerThrottlenumericUpDown3_ValueChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.ThrottletempProbe3 = (double)PowerThrottlenumericUpDown3.Value;
            Properties.Settings.Default.Save();
        }

        private void PowerThrottlenumericUpDown4_ValueChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.ThrottletempProbe4 = (double)PowerThrottlenumericUpDown4.Value;
            Properties.Settings.Default.Save();
        }
    }
}