﻿//TempControl
//Copyright (C) 2006  Casper Kaae Soenderby (casperkaae'at'gmail.com)

//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;
using System.Threading;
using System.Windows.Forms;

namespace TempControl
{
    public class Probe
    {
        private string[] _temp = new string[] {"0.00","0.00","0.00","0.00" };
        private string _versionInfo = "";
        private bool Initialized = false;
        private bool FirstInitializing = true;
        private string Data;
        private SerialPort sp;
        private string _comport = Properties.Settings.Default.ComPort; //default comport 

        //proberties
        public string Temp1
        {
            get { return CalibrateTemp(_temp[0], Properties.Settings.Default.CalibrateProbe1); }
            set{Temp1 = _temp[0];}
        }
        
        public string Temp2
        {
            get { return CalibrateTemp(_temp[1], Properties.Settings.Default.CalibrateProbe2); }
            set{Temp2 = _temp[1];}
        }
        
        public string Temp3
        {
            get { return CalibrateTemp(_temp[2], Properties.Settings.Default.CalibrateProbe3); }
            set{Temp3 = _temp[2];}
        }
        
        public string Temp4
        {
            get { return CalibrateTemp(_temp[3], Properties.Settings.Default.CalibrateProbe4); }
            set{Temp4 = _temp[3];}
        }

        public string VersionInfo
        {
            get { return _versionInfo; }
            set {VersionInfo = _versionInfo;}
        }

        public string Comport
        {
            get { return _comport; }
            set { _comport = value; }
        }





        public void Initialize()
        {
            if (Initialized == false)
            {
                int baud = 2400;
                sp = new SerialPort(_comport, baud);
                sp.DtrEnable = true;
                sp.Parity = Parity.None;
                sp.StopBits = StopBits.Two;
                sp.DataBits = 8;
                sp.Handshake = Handshake.None;
                sp.ReadTimeout = 1000; // doesn't work with lower timeout??
                sp.Open();
                Initialized = true;
            }
            if(FirstInitializing == true)
            {
                    _versionInfo = sp.ReadLine();
                    FirstInitializing = false;
           }

        }
        public void UnInitialize()
        {
            if (Initialized == true)
            {
                sp.DtrEnable = false;
                sp.Close();
                Initialized = false;
            }
        }

        private void ReadTemp() // updates Temps and return a string array with current temps
        {
            string tempsensor;
            if (Initialized == true)
            {
                for (int i = 0; i != 4; ++i)
                {
                    try
                    {
                        Data = sp.ReadLine();
                        //format: "\r2 'temp'" where the number following the r is the probe number
                    }
                    catch(Exception ex)
                    {
                        throw ex;
                    }
                    tempsensor = Data.Remove(0, 1); //gets the number on the probe... 
                    tempsensor = tempsensor.Remove(1);
                    //saves the temp into the correct place i the _temp Array. Didnt use "i" as index because
                    //it might save the temps at the wrong index if some kind of error occurs.
                    _temp[int.Parse(tempsensor)-1] = Data.Remove(0, 5); // removes som " \r 00" from the beginning of the string
                }
            }
            else
            {
                Initialize();
                ReadTemp();
            }
        }

        public void ReadTempLoop()
        {
            while (true)
            {
                try
                {
                    ReadTemp();
                }
                catch (Exception ex)
                {
                    //MessageBox.Show("Could not read Temp. from circuit!, Did it initialize properly?");
                }
            }
        }

        private string CalibrateTemp(string temp, decimal calibratetemp)
        {
            decimal ReturnTemp = (decimal.Parse(temp) / 100) + calibratetemp;
            ReturnTemp.ToString();
            return string.Format("{0:0.00}", ReturnTemp); // padding with two zero's if there is none decimal places. fx 20 is displayed as 20,00 etc.   
        }
    }
}
